/******************************************************************************
*                       DisplayTimeReferenceInformation.c                     *
*******************************************************************************
*
* Description
*    This example displays time reference information.
*
* Requirements:
*    For this example you need a PXI-6683(H) board.
*
* Instructions for Running:
*    1. Update 'deviceName' with the name of your NI-Sync device.
*    2. Run make.
*    3. Run the generated program.
*
******************************************************************************/

#include <nisync.h>
#include <stdio.h>
#include <stdlib.h>

#define niSyncErrorCheck(fCall) if (status = (fCall), status < VI_SUCCESS) \
{  goto Error; } else

ViStatus getSelectedTimeReference (ViSession deviceSession, ViChar** buffer)
{
    ViInt32 selectedTrSize = 0;

    // Get the size of the selected time reference
    selectedTrSize = niSync_GetAttributeViString (deviceSession, VI_NULL,
                                                  NISYNC_ATTR_TIMEREF_SELECTED_TYPE,
                                                  0, VI_NULL);
    if (selectedTrSize <= 0)
    {
        return selectedTrSize;
    }

    // Allocate a buffer of that size
    *buffer = calloc (selectedTrSize, sizeof(ViChar));
    if (*buffer == NULL)
    {
        printf ("Error: unable to allocate memory of size %d\n", selectedTrSize);
        return -1;
    }

    // Get the selected time reference
    ViStatus bufferStatus = niSync_GetAttributeViString (deviceSession, VI_NULL,
                                                         NISYNC_ATTR_TIMEREF_SELECTED_TYPE,
                                                         selectedTrSize, *buffer);
    return bufferStatus;
}

int main (void)
{
    ViRsrc    deviceName = "PXI1Slot2";

    ViSession deviceSession = 0;
    ViInt32   status = 0;
    ViChar    errorString[256];
    ViBoolean isUtcOffsetValid = VI_FALSE;
    ViBoolean isTrPresent = VI_FALSE;
    ViChar*   selectedTr = NULL;
    ViReal64  trOffset = 0;
    ViInt32   utcOffset = 0;
    ViUInt16  timeFractionalNsec = 0;
    ViUInt32  timeSeconds = 0;
    ViUInt32  timeNsec = 0;
    char      c;

    // Open a session to the timing and synchronization module
    niSyncErrorCheck (niSync_init (deviceName, VI_TRUE, VI_FALSE, &deviceSession));

    niSyncErrorCheck (getSelectedTimeReference (deviceSession, &selectedTr));

    niSyncErrorCheck (niSync_GetAttributeViBoolean (deviceSession,
                                                    "",
                                                    NISYNC_ATTR_TIMEREF_PRESENT,
                                                    &isTrPresent));

    niSyncErrorCheck (niSync_GetTime(deviceSession, &timeSeconds,
                                      &timeNsec, &timeFractionalNsec));

    niSyncErrorCheck (niSync_GetAttributeViReal64 (deviceSession,
                                                   "",
                                                   NISYNC_ATTR_TIMEREF_OFFSET,
                                                   &trOffset));

    niSyncErrorCheck (niSync_GetAttributeViBoolean (deviceSession,
                                                    "",
                                                    NISYNC_ATTR_TIMEREF_UTC_OFFSET_VALID,
                                                    &isUtcOffsetValid));

    niSyncErrorCheck (niSync_GetAttributeViInt32 (deviceSession,
                                                  "",
                                                  NISYNC_ATTR_TIMEREF_UTC_OFFSET,
                                                  &utcOffset));

    printf ("The selected time reference is %s\n", selectedTr);
    printf ("Time reference is %s\n",
            isTrPresent ? "present" : "not present");
    printf ("Device time: %d seconds, %d nanoseconds and %d fractional nanoseconds\n",
            timeSeconds, timeNsec, timeFractionalNsec);
    printf ("Offset from time reference is %lf\n", trOffset);
    printf ("UTC offset is %d and is %s\n",
            utcOffset,
            isUtcOffsetValid ? "valid" : "invalid");

Error:
    // Free the buffer memory
    free(selectedTr);

    // Close session
    if (deviceSession)
        niSync_close (deviceSession);

    if (status < VI_SUCCESS)
    {
        niSync_error_message (VI_NULL, status, errorString);

        printf ("Error %d occurred: %s\n", status, errorString);
    }

    printf ("End of program, press Enter to exit.");
    while ((c = getchar ()) != '\n' && c != EOF);
    return status >= VI_SUCCESS ? EXIT_SUCCESS : EXIT_FAILURE;
}
