/******************************************************************************
*                               SetTimeReference.c                            *
*******************************************************************************
*
* Description
*    This example demonstrates how to set the time reference, then persist 
*       configuration.
*
* Requirements:
*    For this example you need a PXI-6683(H) board.
*
* Instructions for Running:
*    1. Update 'deviceName' with the name of your NI-Sync device.
*    2. Update 'trType' with the time reference to set.
*           - for IRIG the IRIG type ('irigType') is required.
*           - for PPS the terminal name ('terminalName') is required.
*    3. Update 'persistConfiguration' to enable or disable persist 
*           configuration.
*    4. Run make.
*    5. Run the generated program.
*
******************************************************************************/

#include <nisync.h>
#include <stdio.h>
#include <stdlib.h>

#define niSyncErrorCheck(fCall) if (status = (fCall), status < VI_SUCCESS) \
{  goto Error; } else

int main (void)
{
    ViRsrc          deviceName = "PXI1Slot2";
    ViInt32         trType = NISYNC_VAL_TIMEREF_GPS;
    ViInt32         irigType = NISYNC_VAL_IRIG_TYPE_IRIGB_DC;
    ViBoolean       persistConfiguration = VI_TRUE;
    ViConstString   terminalName = "PFI0";

    ViSession deviceSession = 0;
    ViInt32   status = 0;
    ViChar    errorString[256];
    char      c;

    // Open a session to the timing and synchronization module
    niSyncErrorCheck (niSync_init (deviceName, VI_TRUE, VI_TRUE, &deviceSession));

    // Set the time reference
    switch (trType)
    {
        case NISYNC_VAL_TIMEREF_1588_ORDINARY_CLOCK:
            printf ("Setting time reference to IEEE 1588\n");
            niSyncErrorCheck (niSync_SetTimeReference1588OrdinaryClock (deviceSession));
            break;
        case NISYNC_VAL_TIMEREF_FREERUNNING:
            printf ("Setting time reference to Free Running\n");
            niSyncErrorCheck (niSync_SetTimeReferenceFreeRunning (deviceSession));
            break;
        case NISYNC_VAL_TIMEREF_GPS:
            printf ("Setting time reference to GPS\n");
            niSyncErrorCheck (niSync_SetTimeReferenceGPS (deviceSession));
            break;
        case NISYNC_VAL_TIMEREF_IRIG:
            printf ("Setting time reference to IRIG\n");
            // For IRIG only PFI0 is supported
            niSyncErrorCheck (niSync_SetTimeReferenceIRIG (deviceSession, irigType, "PFI0"));
            break;
        case NISYNC_VAL_TIMEREF_PPS:
            printf ("Setting time reference to PPS (pulse per second)\n");
            niSyncErrorCheck (niSync_SetTimeReferencePPS (deviceSession, terminalName, VI_FALSE, 0, 0, 0));
            break;
        case NISYNC_VAL_TIMEREF_8021AS:
            printf ("Setting time reference to IEEE 802.1AS\n");
            niSyncErrorCheck (niSync_SetTimeReference8021AS (deviceSession));
            break;
        default:
            printf ("Invalid time reference!\n");
            goto Error;
    }

    // Persist the configuration
    if (persistConfiguration == VI_TRUE)
    {
        niSyncErrorCheck (niSync_PersistConfig (deviceSession));
    }

Error:
    // Close session
    if (deviceSession)
        niSync_close (deviceSession);

    if (status < VI_SUCCESS)
    {
        niSync_error_message (VI_NULL, status, errorString);

        printf ("Error %d occurred: %s\n", status, errorString);
    }

    printf ("End of program, press Enter to exit.");
    while ((c = getchar ()) != '\n' && c != EOF);
    return status >= VI_SUCCESS ? EXIT_SUCCESS : EXIT_FAILURE;
}
